/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.cc;

import hu.midori.kosmos.model.CcBuildStatus;
import hu.midori.kosmos.model.CcProject;
import hu.midori.kosmos.model.CcTest;
import hu.midori.kosmos.protocol.CcService;
import hu.midori.kosmos.server.AbstractKosmosService;
import hu.midori.kosmos.server.AbstractKosmosServiceTest;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.testng.annotations.Configuration;
import org.testng.annotations.Test;

/**
 * Unit test.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class CcServiceTest extends AbstractKosmosServiceTest {
    private CcService service = new CcServiceImpl();

    @Configuration(beforeTestMethod=true)
    @Override
    public void setup() {
        super.setup();

        setupServiceMockEnvironment((AbstractKosmosService)service);
    }

    /** Tests with an invalid path. */
    @Test
    public void testInvalidUrl() {
        assert service.getProjects("http://cruisecontrol.nosuchdomain.com/cc/buildresults/nosuchproduct-testsuite").isEmpty();
    }

    /** Tests with a valid embedded log. */
    @Test
    public void testEmbeddedLog() {
        testLog("http://cruisecontrol.jboss.com/cc/buildresults/jboss-portal-2.2-testsuite");
    }

    /** Tests with a valid external log. */
    @Test
    public void testExternalLog() {
        testLog("http://gmf-dev.borland.com/cruisecontrol/buildresults/nightly");
    }

    /** Tests public JBoss logs. */
    @Test
    public void testJBossLogs() {
        testLog("http://cruisecontrol.jboss.com/cc/buildresults/ejb3-head-testsuite");
        testLog("http://cruisecontrol.jboss.com/cc/buildresults/jboss-cache");
        testLog("http://cruisecontrol.jboss.com/cc/buildresults/jboss-seam-testsuite");
        testLog("http://cruisecontrol.jboss.com/cc/buildresults/jboss-portal-2.4-testsuite");
    }

    /** Tests public Java Desktop logs. */
    @Test
    public void testJavaDesktopLogs() {
        testLog("http://javadesktop.org/cruisecontrol/buildresults/databinding-HEAD");
        testLog("http://javadesktop.org/cruisecontrol/buildresults/jdnc-HEAD");
        testLog("http://javadesktop.org/cruisecontrol/buildresults/swinglabs-demos-HEAD");
        testLog("http://javadesktop.org/cruisecontrol/buildresults/swingx-HEAD");
    }

    /** Tests some other public logs. */
    @Test
    public void testOtherLogs() {
        testLog("http://os.inspiring.nl/cruisecontrol/buildresults/trails");
        testLog("http://www.new-ties.org/cruisecontrol/buildresults/newties");
    }

    /** Tests a single log. */
    @Test
    protected void testLog(String url) {
        List<CcProject> projects = service.getProjects(url);
        assert (projects.size() == 0) || (projects.size() == 1) : String.format("One log should contain exactly zero or one project, not %d", projects.size());
        if(projects.isEmpty())
            return;

        // test project
        CcProject project = projects.get(0);

        assert !StringUtils.isBlank(project.getName());
        assert !StringUtils.isBlank(project.getBuildResultUrl());
        assert !StringUtils.isBlank(project.getBuildLabel());
        assert (project.getStatus() == CcBuildStatus.SUCCEEDED) || (project.getStatus() == CcBuildStatus.FAILED);
        assert project.getLastBuildDate()!= null;

        assert project.getModifications() >= 0;
        assert !StringUtils.isBlank(project.getBuildTime());

        assert project.getTests() >= 0;
        assert project.getTestsSucceded() >= 0;
        assert project.getFailures() >= 0;
        assert project.getErrors() >= 0;
        assert (project.getTestsSucceded() + project.getFailures() + project.getErrors()) == project.getTests();
        assert project.getTestCases().size() == project.getTests();
        for(CcTest testCase : project.getTestCases()) {
            assert testCase.getResult() >= 0;
            assert !StringUtils.isBlank(testCase.getClassName());
            assert !StringUtils.isBlank(testCase.getTestCaseName());
        }

        testZoomableImageLocator(project.getTestsPerCategoryChartLocator());

        assert (project.getSuccessRate() >= 0.0) && (project.getSuccessRate() <= 1.0);
        assert project.getLastBuildAge() >= 0;
    }
}
