/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.sf;

import hu.midori.kosmos.model.SfRelease;
import hu.midori.kosmos.protocol.SfService;
import hu.midori.kosmos.server.AbstractKosmosService;
import hu.midori.kosmos.server.AbstractKosmosServiceTest;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.testng.annotations.Configuration;
import org.testng.annotations.Test;

/**
 * Unit test.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class SfServiceTest extends AbstractKosmosServiceTest {
    private SfService service = new SfServiceImpl();

    @Configuration(beforeTestMethod=true)
    @Override
    public void setup() {
        super.setup();

        setupServiceMockEnvironment((AbstractKosmosService)service);
    }

    /** Tests with invalid URL. */
    @Test
    public void testInvalidUrl() {
        assert service.getFileReleases("http://sourceforge.net/projects/nosuchproject").isEmpty();
    }

    /** Tests with a project that has no release yet. */
    @Test
    public void testProjectWithNoRelease() {
        assert service.getFileReleases("http://sourceforge.net/projects/pixie-perl").isEmpty();
    }

    /** Tests with Kosmos' dependency projects. */
    @Test
    public void testKosmosDependencyProjects() {
        testProject("http://sourceforge.net/projects/cruisecontrol");
        testProject("http://sourceforge.net/projects/displaytag");
        testProject("http://sourceforge.net/projects/jfreechart");
        testProject("http://sourceforge.net/projects/jtidy");
        testProject("http://sourceforge.net/projects/saxon");
        testProject("http://www.sourceforge.net/projects/springframework");
    }

    /** Tests with JBoss product projects. */
    @Test
    public void testJBossProjects() {
        testProject("http://sourceforge.net/projects/jboss");
    }

    /** Tests with Kosmos' tool projects. */
    @Test
    public void testKosmosToolProjects() {
        testProject("http://sourceforge.net/projects/docbook");
    }

    /** Tests various popular SourceForge projects. */
    @Test
    public void testPopularProjects() {
        testProject("http://sourceforge.net/projects/azureus");
        testProject("http://sourceforge.net/projects/gaim");
        testProject("http://sourceforge.net/projects/sevenzip");
        testProject("http://sourceforge.net/projects/pdfcreator");
        testProject("http://sourceforge.net/projects/compiere");
        testProject("http://sourceforge.net/projects/fckeditor");
    }

    /** Tests a single release. */
    @Test
    protected void testProject(String url) {
        List<SfRelease> releases = service.getFileReleases(url);
        assert !releases.isEmpty();

        // test releases
        for(SfRelease release : releases) {
            assert !StringUtils.isBlank(release.getPackageName());
            assert !StringUtils.isBlank(release.getPackageUrl());
            assert !StringUtils.isBlank(release.getVersion());
            assert !StringUtils.isBlank(release.getVersionUrl());
            assert release.getReleaseDate() != null;
            assert release.getAge() >= 0;
            assert release.getAgeInDays() >= 0;
        }
    }
}
